; -------------------------------------------------------------;
;                      MR. ULTRA 2                             ;
; -------------------------------------------------------------;

.sdsctag 0.1, "Mr. Ultra 2", "the second one.", "chris read."

.memorymap
 defaultslot 0
 slotsize $8000 ; 48KB (ROM)
 slot 0 $0000
 slotsize $2000 ; 8KB (RAM)
 slot 1 $2000
 slotsize $100 ; RAM
 slot 2 $dd00
 slotsize $4000 ; ROM
 slot 3 $8000

.endme

.rombankmap
bankstotal 1
banksize $8000 ; 32KB
banks 1
.endro

.equ   vspeed 1            ; players' vertical speed.
.equ   hspeed 3            ; horiz. speed of player and enemy.
.equ   rightb $8c          ; right border of the road.
.equ   leftb $14           ; left border of the road.

; Map of the sprite attribute table (sat) buffer.
; Contains sprites' vertical position (vpos), horizontal posi-
; tion (hpos) and character codes (cc).

.equ lifecountervp $c000          ; first score vpos.
.equ lifecounterhp $c080          ; first score hpos.
.equ lifecountercc $c081          ; first score cc.

.equ mrultravp $c008           ; first Mae vpos.
.equ mrultrahp $c090           ; first Mae hpos.
.equ mrultracc $c091           ; first Mae cc.


.equ cloudvp $c018           ; first player vpos.
.equ cloudhp $c0b0           ; first player hpos.
.equ cloudcc $c0b1           ; first player cc.

.equ enemyvp $c028           ; first Ash vpos.
.equ enemyhp $c0d0           ; first Ash hpos.
.equ enemycc $c0d1           ; first Ash cc.



.equ endspr $c038          ; first unused sprite.

 ; Organize ram - create the sat buffer and variables.

.enum $c000 export         ; export labels to symbol file.

       satbuf dsb 256      ; sprite attribute table buffer.
                           ; see the map for object offsets.
       scroll db           ; vdp scroll register buffer.
       input db            ; input from player 1 controller.
       iflag db            ; frame interrupt flag.
       frame db            ; frame counter.
       status db           ; vdp status (for collision detect.).


       virtualenemyx       db
       lifecounter db
       event	db
       levelnumber	db
       cloudtimer          db
       walkingtimerenemy    db
       automaticwalking     db
       startgame           db
       walkinganim          db
       jumpingtimer         db
       jumping        db
       falling              db
       scrolltimer          db
       mrultrax   db
       mrultray   db
       endlevelanim  db
       enemyyminus8 db
       enemyyplus8  db
       enemyxminus8 db
       enemyxplus8  db
       enemyx     db
       enemyy     db
       cloudx  db
       cloudy  db
       cloudxplus25 db
       enemyyplus24 db
.ende

.bank 0 slot 0
.org 0
       di                  ; disable interrupts.
       im 1                ; interrupt mode 1.
       ld sp,$dff0         ; default stack pointer address.
       jp inigam           ; initialize game.

; Read the vdp status flag at every frame interrupt.
; Sprite collision is read from bit 5 of the status flag.
; Set the frame interrupt flag.

.orga $0038                ; frame interrupt address. 
        ex af,af'           ; save accumulator in its shadow reg.
        in a,$bf            ; get vdp status / satisfy interrupt.
        ld (status),a       ; save vdp status in ram. 
        ld a,1              ; load accumulator with raised flag. 
        ld (iflag),a        ; set interrupt flag. 
        ex af,af'           ; restore accumulator. 
        ei                  ; enable interrupts.
        reti                ; return from interrupt. 

; Disable the pause button

.orga $0066                ; pause button interrupt.
       retn                ; disable pause button.

; Initialize game.
; Overwrite the first 4 kb of ram with zeroes.

inigam
       call PSGStop        ; turn off the music.
       ld hl,$c000         ; point to beginning of ram.
       ld bc,$1000         ; 4 kb to fill.
       ld a,0              ; with value 0.
       call mfill          ; do it!

; Use the initialized ram to clean all of vram.

       ld hl,$0000         ; prepare vram for data at $0000.
       call vrampr
       ld b,4              ; write 4 x 4 kb = 16 kb.
-      push bc             ; save the counter.
       ld hl,$c000         ; source = freshly initialized ram.
       ld bc,$1000         ; 4 kb of zeroes.
       call vramwr         ; purge vram.
       pop bc              ; retrieve counter.
       djnz -

; Initialize the VDP registers.

       ld hl,regdat        ; point to register init data.
       ld b,11             ; 11 bytes of register data.
       ld c,$80            ; VDP register command byte.

-:     ld a,(hl)           ; load one byte of data into A.
       out ($bf),a         ; output data to VDP command port.
       ld a,c              ; load the command byte.
       out ($bf),a         ; output it to the VDP command port.
       inc hl              ; inc. pointer to next byte of data.
       inc c               ; inc. command byte to next register.
       djnz -              ; jump back to '-' if b > 0.

 ; Initialize PSGLib.

       call PSGInit        ; credit goes to sverx!

; Load various assets into vram.
       ld hl,$2000         ; first tile @ index 256.
       call vrampr         ; prepare vram.
       ld hl,mrultratile1        ; player car tile data.
       ld bc,16*32         ; 16 tiles, 32 bytes each.
       call vramwr         ; write player car tiles to vram.

       ld hl,$c000
       call vrampr         ; prepare vram.
       ld hl,bgpal         ; background palette.
       ld bc,32
       call vramwr         ; set background palette.

       ld hl,$c020
       call vrampr         ; prepare vram.
       ld hl,palclouds         ; background palette.
       ld bc,32
       call vramwr         ; set background palette.

       xor a

       ld a,1            ;
       ld (cloudtimer),a

       ld a,220            ;
       ld (mrultray),a

       ld a,32            ;
       ld (cloudy),a
       ld a,20            ;
       ld (cloudx),a
       ld a,0
       ld (startgame),a

; Prepare for the title screen loop.

prepti
; Load title screen specific assets into vram.

       ld hl,$0000         ; first tile @ index 0.
       call vrampr         ; prepare vram.
       ld hl,titile        ; title screen tile data.
       ld bc,115*32         ; 115 tiles, each tiles is 32 bytes.
       call vramwr         ; write title screen tiles to vram.

       ld hl,$3800         ; point to name table.
       call vrampr         ; prepare vram.
       ld hl,timap         ; title screen tile map data.
       ld bc,32*24*2       ; 32 x 24 tiles, each is 2 bytes.
       call vramwr         ; write name table to vram.

       ld de,mrultracc         ; point to player cc in buffer.
       ld hl,mrultratilemap        ; point to player car graphics.
       call mrultra1cc          ; set the char codes for player car.

       ld de,cloudcc         ; point to player cc in buffer.
       ld hl,cloudmap        ; point to player car graphics.
       call carcc          ; set the char codes for player car.

; Create sprite data in the sat buffer for the hiscore sprites,
; and load the updated buffer into vram (screen is off now).

       call upbuf          ; update sat buffer.
       call ldsat          ; load sat in vram with buffer.

; Turn screen on.
       xor a
       ld b,9              ; reset scroll value.
       call setreg         ; set register 1.

       ld a,%11100000      ; enable screen
       ld b,1              ; register 1.
       call setreg         ; set register.

; Display title screen and wait for player 1 button 1 press.
       ld hl,$2000         ; first tile @ index 256.
       call vrampr         ; prepare vram.
       ld hl,mrultratile1        ; player car tile data.
       ld bc,4*32          ; 4 tiles, 32 bytes each.
       call vramwr         ; write player tiles to vram.

       ei                  ; turn interrupts on.

; Play title screen tune.
       ld hl,titune        ; load title screen tune.
       call PSGPlay        ; play it.

titlescreenstuff
       halt                ; start main loop with vblank.

       ld a,(startgame)
       cp 0
       jp c,titlescreenstuff2

       in     a,($00)
       ;cpl                    ; invert logic so 1 = pressed
       and    %10000000

       jp nz,titlescreenstuff2

       ld a,1           ;
       ld (startgame),a

       ld a,115           ;
       ld (mrultray),a

       ld a,30          ;
       ld (mrultrax),a

titlescreenstuff2
        call ldsat
       call PSGFrame

       ld a,(cloudtimer)
       add a,1
       ld (cloudtimer),a

       ld a,(cloudtimer)
       cp 10
       jp nc,move_cloud

titlescreenstuff3
       ld a,(startgame)
       cp 1
       jp nc,walking_animation

            call upbuf
               call upbuf2

       jp titlescreenstuff

move_cloud
       ld a,(cloudx)
       add a,1
       ld (cloudx),a

       ld a,0
       ld (cloudtimer),a

              call upbuf          ; update sat buffer.
                 call upbuf2
              jp titlescreenstuff3

walking_animation
       ld a,(mrultrax)
       add a,1
       ld (mrultrax),a

       ld a,(mrultrax)
       cp 205
       jp nc,start_game

       ld a,(walkinganim)
       add a,1
       ld (walkinganim),a

       ld a,(walkinganim)
       cp 5
       jp c,walking_anim3

       ld a,(walkinganim)
       cp 9
       jp z,walking_anim2

       ld hl,$2000
       call  vrampr        ; prepare vram at the above address.
       ld hl,mrultratile1
       ld bc,4*32         ; 4 tiles, each tile is 32 bytes.
       call vramwr         ; write characters to vram.

            call upbuf          ; update sat buffer.
               call upbuf2

       jp titlescreenstuff

walking_anim2
       ld a,0            ;
       ld (walkinganim),a

walking_anim3
       ld hl,$2000         ; first tile @ index $256.
       call vrampr        ; prepare vram at the above address.
       ld hl,mrultratile2
       ld bc,4*32         ; 4 tiles, each tile is 32 bytes.
       call vramwr         ; write characters to vram.

       call upbuf          ; update sat buffer.
               call upbuf2

       jp titlescreenstuff

start_game
; Setup hiscore and score for main loop and remove terminator.

       ld hl,initsc        ; point to score init data.
       ld de,lifecountervp        ; target the score sat buffer.
       ld bc,5             ; write 8 vpos bytes.
       ldir                ; do it.
       ld hl,initsc+5      ; point to score init data #2.
       ld de,lifecounterhp        ; target buffer again.
       ld bc,8
       ldir

       ld hl,lifecounter      ; point to ones digit in life counter
       ld b,3              ; we want to add 3 to this digit. (gives 4 lives at the start.)
       ld a,(hl)           ; get the value of the digit.
       add a,b             ; add the amount to this value.
       ld (hl),a           ; put updated digit back in string.

start_level_1
       ld a,3         ;
       ld (levelnumber),a

       ld hl,$c000
       call vrampr         ; prepare vram.
       ld hl,level3pal         ; background palette.
       ld bc,16
       call vramwr         ; set background palette.

       ld hl,$0000         ; first tile @ index 0.
       call vrampr         ; prepare vram.
       ld hl,level3tile        ; title screen tile data.
       ld bc,73*32         ; 73 tiles, each tiles is 32 bytes.
       call vramwr         ; write title screen tiles to vram.

       ld hl,$3800         ; point to name table.
       call vrampr         ; prepare vram.
       ld hl,level3map         ; title screen tile map data.
       ld bc,32*24*2       ; 32 x 24 tiles, each is 2 bytes.
       call vramwr         ; write name table to vram.

       ld hl,$2000         ; first tile @ index $140.
       call vrampr        ; prepare vram at the above address.
       ld hl,mrultratile1
       ld bc,4*32         ; 4 tiles, each tile is 32 bytes.
       call vramwr         ; write red characters to vram.

         ld hl,$c020
       call vrampr         ; prepare vram.
       ld hl,palclouds         ; background palette.
       ld bc,32
       call vramwr         ; set background palette.

start_new_level
       call PSGStop        ; turn off the music.

pick_out_which_tune_to_play
       ld a,(levelnumber)
       cp 2
       jp z,play_level2tune

        ld a,(levelnumber)
       cp 3
       jp z,play_level3tune

       ld hl,level1tune        ; load tune.
       call PSGPlay        ; play it.

       jp start_new_level_2

play_level2tune
       ld hl,level2tune        ; load tune.
       call PSGPlay        ; play it.
       jp start_new_level_2

play_level3tune
       ld hl,level3tune        ; load tune.
       call PSGPlay        ; play it.

start_new_level_2
       ld hl,$2600
       call  vrampr        ; prepare vram at the above address.
       ld hl,blchar        ; the blue characters - for hiscore.
       ld bc,10*32         ; 10 tiles, each tile is 32 bytes.
       call vramwr         ; write blue character tils to vram.

       ld a,0          ;
       ld (jumping),a

       ld a,0          ;
       ld (endlevelanim),a

       ld a,0          ;
       ld (falling),a

       ld a,0          ;
       ld (scrolltimer),a       

       ld a,0          ;
       ld (jumpingtimer),a

       ld a,1         ;
       ld (event),a

       ld a,0          ;
       ld (automaticwalking),a

       ld a,0          ;
       ld (walkingtimerenemy),a          ;

       ld a,2          ;
       ld (startgame),a          ;

       ld a,170           ;
       ld (cloudx),a
       
       ld a,199          ;
       ld (cloudxplus25),a

       ld a,143         ;
       ld (cloudy),a

       ld a,60          ;
       ld (mrultrax),a
       ld a,113           ;
       ld (mrultray),a

       ld a,220          ;
       ld (virtualenemyx),a          ; virtualenemyx is the variable that tracks
                                     ; mr. ultra's movement when the enemy has gone
                                     ; off screen and before the next event happens.
                                     ; when virtualenemyx=10, then the next event SHOULD happen.
                                     ; virtualenemyx should only subtract when player presses right.
                                     ; and when it gets down to 10, make it be 220 again and
                                     ; increase the event by one.

       ld a,220          ;
       ld (enemyx),a
       ld a,113           ;
       ld (enemyy),a

       ld de,cloudcc
       ld hl,level1holemap
       call carcc

       ld de,mrultracc
       ld hl,mrultratilemap2
       call mrultra1cc

       ld de,enemycc
       ld hl,enemymap
       call carcc

main_game_loop
       halt

       ; where the game logic in game starts. Cloud stuff shares with hole logic, while
       ; enemy stuff is for the various vegetable enemies.

       ; as it stands now, a fourth stalk of broccoli comes out but disappears soon after and a hole
       ; takes its place. this obviously is not what I want to have happen.

       ld a,(cloudx)
       add a,24
       ld (cloudxplus25),a

       call PSGFrame       ; handle music.

       ; collision detection for enemy

       ld a,(enemyy)
       ld b, 24
       sub b
       ld (enemyyminus8),a

       ld de,mrultray
       ld hl,enemyyminus8

       ld a,(de)
       cp (hl)
       jp c,test_hole_coll

       ld a,(enemyy)
       add a,5
       ld (enemyyplus8),a

       ld de,mrultray
       ld hl,enemyyplus8

       ld a,(de)
       cp (hl)
       jp nc,test_hole_coll

       ; test enemyx collision
       ld a,(enemyx)
       ld b, 5
       sub b
       ld (enemyxminus8),a

       ld de,mrultrax
       ld hl,enemyxminus8

       ld a,(de)
       cp (hl)
       jp c,test_hole_coll

       ld a,(enemyx)
       add a,8
       ld (enemyxplus8),a

       ld de,mrultrax
       ld hl,enemyxplus8

       ld a,(de)
       cp (hl)
       jp nc,test_hole_coll

       ld a,2
       ld (falling),a

test_hole_coll
       ld a,(mrultray)
       cp 113
       jp c,donttestanything

       ld de,mrultrax
       ld hl,cloudx

       ld a,(de)
       cp (hl)
       jp c,donttestanything

       ld a,(cloudx)
       add a,32
       ld (cloudxplus25),a

       ld de,mrultrax
       ld hl,cloudxplus25

       ld a,(de)
       cp (hl)
       jp nc,donttestanything

       ld a,2          ;
       ld (falling),a

donttestanything
       call upbufgame
       call upbufgameenemy

donttestanything5
       ld a,(virtualenemyx)
       cp 10
       jp z,get_event_level_1

       ld a,(automaticwalking)
       cp 1
       jp z,donttestanything52

donttestanything53
       call upbufgamehole
       
donttestanything52
       call ldsat

       ld a,(endlevelanim)
       cp 1
       jp z,theendoflevel

       ld a,(falling)
       cp 2
       jp z,fallingtodeathcode2
     
       call getkey         ; read controller port.
       ld a,(input)        ; read input from ram mirror.
       bit 3,a             ; is right key pressed?
       jp z,walk_right

       ld a,(enemyx)
       cp 220
       jp z,main_game_loop_22

       ld a,(automaticwalking)
       cp 2
       jp z,main_game_loop_22

       ld a,(enemyx)
       add a,1
       ld (enemyx),a

main_game_loop_2
       ld a,(automaticwalking)
       cp 1
       jp z,switchtocarrot

main_game_loop_22
       ld a,(falling)
       cp 1
       jp z,fallingcode     

       call getkey         ; read controller port into ram.
       ld a,(input)        ; read input from ram mirror.
       bit 4,a             ; is button 1 pressed?
       jp z,jumpingcode

       ld a,(jumping)
       cp 1
       jp z,fallingcode

main_game_loop_2a
                    
main_game_loop_3
       jp main_game_loop

walk_right
       ld a,(scrolltimer)
       add a,1
       ld (scrolltimer),a

       ld a,(scrolltimer)
       cp 1
       jp z,scrollingcode

stopscrolling

walkright2
       jp walking_animation_ingame

get_event_level_1
       ld a,10
       ld (virtualenemyx),a

       ld a,(enemyx)
       cp 10
       jp z,donttestanything52

       ld a,(cloudx)
       cp 10
       jp z,donttestanything53

new____
       ld a,220         ;
       ld (virtualenemyx),a

       ld a,(event)
       add a,1
       ld (event),a

       ld a,(levelnumber)
       cp 2
       jp z,get_event_level_2

       ld a,(levelnumber)
       cp 3
       jp z,get_event_level_2

       jp move_event

get_event_level_3
       ld a,(event)
       cp 1      
       jp z,switchtocarrot

       ld a,(event)          
       cp 2        
       jp z,dontautomaticallyswitchtocarrot

       ld a,(event)          
       cp 4        
       jp z,switchtocarrot

       ld a,(event)          
       cp 6       
       jp z,switchtocarrot

       ld a,(event)
       cp 8       
       jp z,switchtocarrot

       ld a,(event)          
       cp 9       
       jp z,dontautomaticallyswitchtocarrot

       ld a,(event)          
       cp 11       
       jp z,switchtocarrot

       ld a,(event)          
       cp 12    
       jp z,dontautomaticallyswitchtocarrot

       ld a,(event)          
       cp 14    
       jp z,switchtocarrot

       ld a,(event)
       cp 15    
       jp z,dontautomaticallyswitchtocarrot

       ld a,(event)          
       cp 17    
       jp z,theendoflevel

       jp switchtohole

get_event_level_2
       ld a,(event)
       cp 2        
       jp z,switchtocarrot

       ld a,(event)          
       cp 3        
       jp z,dontautomaticallyswitchtocarrot

       ld a,(event)          
       cp 6        
       jp z,switchtocarrot

       ld a,(event)          
       cp 7       
       jp z,dontautomaticallyswitchtocarrot

       ld a,(event)          
       cp 9       
       jp z,switchtocarrot

       ld a,(event)          
       cp 10       
       jp z,dontautomaticallyswitchtocarrot

       ld a,(event)          
       cp 11       
       jp z,dontautomaticallyswitchtocarrot

       ld a,(event)          
       cp 13    
       jp z,switchtocarrot

       ld a,(event)          
       cp 14    
       jp z,dontautomaticallyswitchtocarrot

       ld a,(event)
       cp 16    
       jp z,switchtocarrot

       ld a,(event)          
       cp 17    
       jp z,theendoflevel

       jp switchtohole

move_event
       ld a,(event)          
       cp 2        
       jp z,switchtocarrot

       ld a,(event)          
       cp 4        
       jp z,switchtocarrot

       ld a,(event)          
       cp 5        
       jp z,dontautomaticallyswitchtocarrot

       ld a,(event)          
       cp 7       
       jp z,switchtocarrot

       ld a,(event)          
       cp 9        
       jp z,switchtocarrot

       ld a,(event)          
       cp 10       
       jp z,dontautomaticallyswitchtocarrot

       ld a,(event)          
       cp 13       
       jp z,switchtocarrot

       ld a,(event)          
       cp 15       
       jp z,switchtocarrot

       ld a,(event)          
       cp 16    
       jp z,dontautomaticallyswitchtocarrot

       ld a,(event)          
       cp 17    
       jp z,theendoflevel

switchtohole
       ld a,0         ;
       ld (automaticwalking),a
       ld a,220          ;
       ld (enemyx),a
       ld a,113           ;
       ld (enemyy),a

       ld de,cloudcc
       ld hl,level1holemap
       call carcc

       ld hl,$c020
       call vrampr
       ld hl,palclouds
       ld bc,32
       call vramwr

       jp main_game_loop_2

dontautomaticallyswitchtocarrot
       ld a,2        ;
       ld (automaticwalking),a
       jp main_game_loop_22

switchtocarrot
       ld a,(enemyx)
       ld b, 2
       sub b
       ld (enemyx),a

       ld a,1          ;
       ld (automaticwalking),a
       ld a,220          ;
       ld (cloudx),a
       ld a,143           ;
       ld (cloudy),a

       ld a,(walkingtimerenemy)
       add a,1
       ld (walkingtimerenemy),a

       ld a,(walkingtimerenemy)          
       cp 11         
       jp z,walking_anim33

       ld a,(walkingtimerenemy)          
       cp 21         
       jp z,walking_anim22

       jp main_game_loop_22

walking_anim33
       ld a,(levelnumber)          
       cp 2       
       jp z,eggplant33

       ld a,(levelnumber)          
       cp 3       
       jp z,broccoli33

       ld hl,$2200
       call vrampr         ; prepare vram.
       ld hl,carrottile2      ; hole tile data.
       ld bc,4*32          ; 4 tiles, 32 bytes each.
       call vramwr

      jp main_game_loop_22

eggplant33
       ld hl,$c020         ; color bank 1, color 0.
       call vrampr         ; prepare vram.
       ld hl,palclouds2         ; background palette.
       ld bc,32
       call vramwr         ; set background palette.

       ld hl,$2200
       call vrampr         ; prepare vram.
       ld hl,eggplanttile2
       ld bc,4*32          ; 4 tiles, 32 bytes each.
       call vramwr

      jp main_game_loop_22

broccoli33
       ld hl,$c020
       call vrampr         ; prepare vram.
       ld hl,palclouds2         ; background palette.
       ld bc,32
       call vramwr         ; set background palette.

       ld hl,$2200
       call vrampr         ; prepare vram.
       ld hl,broccolitile2      ; hole tile data.
       ld bc,4*32
       call vramwr

      jp main_game_loop_22

walking_anim22
       ld a,0
       ld (walkingtimerenemy),a

       ld a,(levelnumber)          
       cp 2       
       jp z,eggplant22
       
       ld a,(levelnumber)          
       cp 3       
       jp z,broccoli22

       ld hl,$2200
       call vrampr         ; prepare vram.
       ld hl,carrottile1      ; hole tile data.
       ld bc,4*32
       call vramwr

      jp main_game_loop_22

eggplant22
       ld hl,$c020
       call vrampr         ; prepare vram.
       ld hl,palclouds2         ; background palette.
       ld bc,32
       call vramwr         ; set background palette.

       ld hl,$2200
       call vrampr         ; prepare vram.
       ld hl,eggplanttile1      ; hole tile data.
       ld bc,4*32
       call vramwr         ; write enemy car tiles to vram.

       jp main_game_loop_22

broccoli22
       ld hl,$c020
       call vrampr         ; prepare vram.
       ld hl,palclouds2         ; background palette.
       ld bc,32
       call vramwr         ; set background palette.

       ld hl,$2200
       call vrampr         ; prepare vram.
       ld hl,broccolitile1      ; hole tile data.
       ld bc,4*32
       call vramwr

       jp main_game_loop_22

walking_animation_ingame   
       ld a,(walkinganim)
       add a,1
       ld (walkinganim),a

       ld a,(walkinganim)          
       cp 5         
       jp c,walking_anim3_ingame

       ld a,(walkinganim)          
       cp 9         
       jp z,walking_anim2_ingame

       ld hl,$2000
       call  vrampr        ; prepare vram at the above address.
       ld hl,mrultratile1
       ld bc,4*32
       call vramwr

       jp main_game_loop_2

walking_anim2_ingame
       ld a,0            ;
       ld (walkinganim),a

walking_anim3_ingame
       ld hl,$2000         ; first tile @ index $256.
       call vrampr        ; prepare vram at the above address.
       ld hl,mrultratile2
       ld bc,4*32
       call vramwr

       jp main_game_loop_2

jumpingcode
       ld a,1          ;
       ld (jumping),a

       ld hl,$2000         ; first tile @ index $256.
       call vrampr        ; prepare vram at the above address.
       ld hl,mrultratile2
       ld bc,4*32
       call vramwr

       ld a,(jumpingtimer)
       add a,1
       ld (jumpingtimer),a

       ld a,(jumpingtimer)          
       cp 50         
       jp nc,fallingcode

       ld a,(mrultray)
       ld b, 1
       sub b
       ld (mrultray),a

       jp main_game_loop_3

fallingcode
       ld a,1          ;
       ld (falling),a      

       ld hl,$2000         ; first tile @ index $256.
       call vrampr        ; prepare vram at the above address.
       ld hl,mrultratile2
       ld bc,4*32
       call vramwr

       ld a,0          ;
       ld (jumping),a

       ld a,(mrultray)
       add a,1
       ld (mrultray),a

       ld a,(mrultray)          
       cp 113        
       jp nc,stopfalling

       jp main_game_loop_3

fallingtodeathcode
       ld a,(mrultray)
       add a,1
       ld (mrultray),a

       ld a,(mrultray)          
       cp 172         
       jp nc,dead

       jp main_game_loop

dead
       ld a,(lifecounter)          
       cp 0         
       jp z,inigam

       ld hl,lifecounter
       ld a,(hl)
       ld b, 1
       sub b
       ld (hl),a

       jp start_new_level

fallingtodeathcode2
       ld a,2
       ld (falling),a

       ld a,(mrultray)
       add a,1
       ld (mrultray),a

       ld a,(mrultray)          
       cp 172      
       jp nc,dead

       jp main_game_loop

stopfalling
       ld a,0
       ld (falling),a       

       ld a,0          ;
       ld (jumpingtimer),a               

       jp main_game_loop_2a

scrollingcode
       ld a,(scroll)       ; 1-byte scroll reg. buffer in ram.
       ld b,8              ; target VDP register 9 (v-scroll).
       call setreg         ; now vdp register = buffer, and the
                           ; screen scrolls accordingly.

       ld a,(scroll)       ; get scroll buffer value.
       sub vspeed          ; subtract vertical speed.
       ld (scroll),a       ; update scroll buffer.


       ld a,(virtualenemyx)          
       cp 10       
       jp z,donttestanything54

       ld a,(virtualenemyx)
       ld b, 1
       sub b
       ld (virtualenemyx),a

donttestanything54
       ld a,(automaticwalking)          
       cp 0     
       jp nz,scrollingcode2

       ld a,(cloudx)
       ld b, 1
       sub b
       ld (cloudx),a

scrollingcode2
       ld a,0
       ld (scrolltimer),a

       ld a,(automaticwalking)
       cp 2      
       jp nz,stopscrolling

       ld a,1        ;
       ld (automaticwalking),a

       jp stopscrolling

theendoflevel
       ld a,220
       ld (enemyx),a

       ld a,1
       ld (endlevelanim),a

       ld a,(mrultrax)
       add a,1
       ld (mrultrax),a

       ld a,(mrultrax)          
       cp 200      
       jp nc,level_2_stuff

       ld a,(falling)          
       cp 1         
       jp z,fallingcode

       jp walking_animation_ingame

level_2_stuff
       ld a,(levelnumber)
       cp 2
       jp z,level_3_stuff

       ld a,2         ;
       ld (levelnumber),a

       ld hl,$c020
       call vrampr         ; prepare vram.
       ld hl,palclouds2         ; background palette.
       ld bc,32
       call vramwr         ; set background palette.

       ld hl,$c000
       call vrampr         ; prepare vram.
       ld hl,level2pal         ; background palette.
       ld bc,32
       call vramwr         ; set background palette.

       ld hl,$0000         ; first tile @ index 0.
       call vrampr         ; prepare vram.
       ld hl,level2tile        ; title screen tile data.
       ld bc,73*32         ; 73 tiles, each tiles is 32 bytes.
       call vramwr         ; write title screen tiles to vram.

       ld hl,$3800         ; point to name table.
       call vrampr         ; prepare vram.
       ld hl,level2map     ; title screen tile map data.
       ld bc,32*24*2       ; 32 x 24 tiles, each is 2 bytes.
       call vramwr         ; write name table to vram.
       
       jp start_new_level

level_3_stuff
       ld a,3         ;
       ld (levelnumber),a

       ld hl,$c000
       call vrampr         ; prepare vram.
       ld hl,level3pal         ; background palette.
       ld bc,16
       call vramwr         ; set background palette.

       ld hl,$0000         ; first tile @ index 0.
       call vrampr         ; prepare vram.
       ld hl,level3tile        ; title screen tile data.
       ld bc,73*32         ; 73 tiles, each tiles is 32 bytes.
       call vramwr         ; write title screen tiles to vram.

       ld hl,$3800         ; point to name table.
       call vrampr         ; prepare vram.
       ld hl,level3map         ; title screen tile map data.
       ld bc,32*24*2       ; 32 x 24 tiles, each is 2 bytes.
       call vramwr         ; write name table to vram.

       jp start_new_level



; --------------------------------------------------------------
; SUBROUTINES
; --------------------------------------------------------------
; PREPARE VRAM.
; Set up vdp to recieve data at vram address in HL.

vrampr push af
       ld a,l
       out ($bf),a
       ld a,h
       or $40
       out ($bf),a
       pop af
       ret

; --------------------------------------------------------------
; WRITE TO VRAM
; Write BC amount of bytes from data source pointed to by HL.
; Tip: Use vrampr before calling.

vramwr ld a,(hl)
       out ($be),a
       inc hl
       dec bc
       ld a,c
       or b
       jp nz,vramwr
       ret

; --------------------------------------------------------------
; LOAD SPRITE ATTRIBUTE TABLE
; Load data into sprite attribute table (SAT) from the buffer.

ldsat  ld hl,$3f00         ; point to start of SAT in vram.
       call vrampr         ; prepare vram to recieve data.
       ld b,255            ; amount of bytes to output.
       ld c,$be            ; destination is vdp data port.
       ld hl,satbuf        ; source is start of sat buffer.
       otir                ; output buffer to vdp.
       ret

; --------------------------------------------------------------
; UPDATE SAT BUFFER
; Generate vpos, hpos and cc data for the sprites that make up
; each of the cars (player, Mae and Ash).
; Also generate char code (cc) data from hiscore and score.

; Generate sat buffer data from player's x,y coordinates.

upbuf                             
       ld a,(mrultray)          ; load mr. ultra's current y-coordinate.
       ld hl,mrultravp         ; point to sat buffer.
       call mrultray2           ; refresh buffer according to y.

       ld a,(mrultrax)          ; load mr. ultra's current x-coordinate.
       ld hl,mrultrahp         ; point to sat buffer.
       call mrultrax2         ; refresh buffer according to x.

       ld a,(cloudy)          ; load cloud's current y-coordinate.
       ld hl,cloudvp         ; point to sat buffer.
       call cloudy2           ; refresh buffer according to y.

       ld a,(cloudx)          ; load cloud's current x-coordinate.
       ld hl,cloudhp         ; point to sat buffer.
       call cloudx2         ; refresh buffer according to x.

        ret
        
upbuf2
       ld a,(cloudy)          ; load cloud's current y-coordinate.
       ld hl,cloudvp         ; point to sat buffer.
       call cloudy2           ; refresh buffer according to y.

       ld a,(cloudx)          ; load cloud's current x-coordinate.
       ld hl,cloudhp         ; point to sat buffer.
       call cloudx2         ; refresh buffer according to x.

       ret

upbufgame
       ld a,(mrultray)          ; load mr. ultra's current y-coordinate.
       ld hl,mrultravp         ; point to sat buffer.
       call mrultray2           ; refresh buffer according to y.

       ld a,(mrultrax)          ; load mr. ultra's current x-coordinate.
       ld hl,mrultrahp         ; point to sat buffer.
       call mrultrax2         ; refresh buffer according to x.

       ld hl,lifecounter
       ld de,lifecountercc

       ld a,(hl)
       add a,48
       ld (de),a
       inc hl
       inc de
       inc de

       ret

upbufgamehole
       ld a,(cloudy)          ; load hole's current y-coordinate.
       ld hl,cloudvp         ; point to sat buffer.
       call holey2           ; refresh buffer according to y.

       ld a,(cloudx)          ; load hole's current x-coordinate.
       ld hl,cloudhp         ; point to sat buffer.
       call holex2         ; refresh buffer according to x.
            ret


upbufgameenemy
       ld a,(enemyy)          ; load enemy's current y-coordinate.
       ld hl,enemyvp         ; point to sat buffer.
       call enemyy2           ; refresh buffer according to y.

       ld a,(enemyx)          ; load enemy's current x-coordinate.
       ld hl,enemyhp         ; point to sat buffer.
       call enemyx2         ; refresh buffer according to x.

       ret


cloudy2    ld c,a              ; save hpos in C
       .rept 2             ; wladx: Repeat code twice.
       ld a,c              ; load hpos into A
       ld b,2            ; loop: Repeat twice.
-      ld (hl),a           ; write value to buffer at address.
       inc hl              ; skip over the char code byte.
       inc hl              ; point to next hpos byte in buffer.
       add a,0             ; add nothing
       djnz -              ; jump back
       .endr               ; end of wladx repeat directive.
       ret


holey2  ld b,4
-      push af             ; a row of 4 tiles share the same y,
       push af             ; so here the y's are saved on stack.
       push af
       push af
       add a,8             ; next row is 8 pixels below.
       djnz -              ; make 4 consecutive rows.

       ld de,15            ; load buffer offset into DE.
       add hl,de           ; add buffer offset to HL.
       ld b,16             ; we need to update 16 bytes.
-      pop af              ; get saved y from stack.
       ld (hl),a           ; write it to the buffer.
       dec hl              ; point to previous byte.
       djnz -              ; backwards from vpos+15 to vpos+0.
       ret


cloudx2   ld c,a              ; save hpos in C
       .rept 3            ; wladx: Repeat code 3 times.
       ld a,c              ; load hpos into A

       ld b,3             ; loop: Repeat 3 times.
-      ld (hl),a           ; write value to buffer at address.
       inc hl              ; skip over the char code byte.
       inc hl              ; point to next hpos byte in buffer.
       add a,6            ; add 6 (to make that certain look)
       djnz -              ; jump back
       .endr               ; end of wladx repeat directive.
       ret

holex2     ld c,a              ; save hpos in C
       .rept 4             ; wladx: Repeat code four times.
       ld a,c              ; load hpos into A
       ld b,4              ; loop: Repeat four times.
-      ld (hl),a           ; write value to buffer at address.
       inc hl              ; skip over the char code byte.
       inc hl              ; point to next hpos byte in buffer.
       add a,8             ; add 8 (a tile's width in pixels).
       djnz -              ; jump back
       .endr               ; end of wladx repeat directive.
       ret

carcc ld bc,16
-      ldi
       inc de
       ld a,b
       or c
       jp nz,-
       ret

cloud1cc ld bc,16
-      ldi
       inc de
       ld a,b
       or c
       jp nz,-
       ret

mrultra1cc ld bc,16
-      ldi
       inc de
       ld a,b
       or c
       jp nz,-
       ret

enemyy2    ld c,a              ; save hpos in C
       .rept 4             ; wladx: Repeat code four times.
       ld a,c              ; load hpos into A
       ld b,4             ; loop: Repeat four times.
-      ld (hl),a           ; write value to buffer at address.
       inc hl              ; skip over the char code byte.
       inc hl              ; point to next hpos byte in buffer.
       add a,8             ; add 8 (a tile's width in pixels).
       djnz -              ; jump back
       .endr               ; end of wladx repeat directive.
       ret


enemyx2   ld c,a              ; save hpos in C
       .rept 3             ; wladx: Repeat code three times.
       ld a,c              ; load hpos into A

       ld b,3             ; loop: Repeat three times.
-      ld (hl),a           ; write value to buffer at address.
       inc hl              ; skip over the char code byte.
       inc hl              ; point to next hpos byte in buffer.
       add a,0            ; add 0
       djnz -              ; jump back
       .endr               ; end of wladx repeat directive.
       ret

mrultray2    ld c,a              ; save hpos in C
       .rept 4             ; wladx: Repeat code four times.
       ld a,c              ; load hpos into A
       ld b,4             ; loop: Repeat four times.
-      ld (hl),a           ; write value to buffer at address.
       inc hl              ; skip over the char code byte.
       inc hl              ; point to next hpos byte in buffer.
       add a,8             ; add 8 (a tile's width in pixels).
       djnz -              ; jump back
       .endr               ; end of wladx repeat directive.
       ret

mrultrax2   ld c,a              ; save hpos in C
       .rept 3             ; wladx: Repeat code three times.
       ld a,c              ; load hpos into A

       ld b,3             ; loop: Repeat three times.
-      ld (hl),a           ; write value to buffer at address.
       inc hl              ; skip over the char code byte.
       inc hl              ; point to next hpos byte in buffer.
       add a,0            ; add nothing.
       djnz -              ; jump back
       .endr               ; end of wladx repeat directive.
       ret

; --------------------------------------------------------------


; --------------------------------------------------------------
; SET VDP REGISTER.
; Write to target register.
; A = byte to be loaded into vdp register.
; B = target register 0-10.

setreg out ($bf),a         ; output command word 1/2.
       ld a,$80
       or b
       out ($bf),a         ; output command word 2/2.
       ret

; --------------------------------------------------------------
; GET KEYS.
; Read player 1 keys (port $dc) into ram mirror (input).

getkey in a,$dc            ; read player 1 input port $dc.
       ld (input),a        ; let variable mirror port status.
       ret

; --------------------------------------------------------------
; MEMORY FILL.
; HL = base address, BC = area size, A = fill byte.

mfill  ld (hl),a           ; load filler byte to base address.
       ld d,h              ; make DE = HL.
       ld e,l
       inc de              ; increment DE to HL + 1.
       dec bc              ; decrement counter.
       ld a,b              ; was BC = 0001 to begin with?
       or c
       ret z               ; yes - then just return.
       ldir                ; else - write filler byte BC times,
                           ; while incrementing DE and HL...
       ret

; --------------------------------------------------------------
; SET COLOR.
; A = color index, B = color value (intensity).
setcol out ($bf),a
       ld a,%11000000
       out ($bf),a
       ld a,b
       out ($be),a
       ret

; --------------------------------------------------------------
; Wait for vertical blanking phase.
wait   ld a,(iflag)        ; load frame interrupt flag.
       or a                ; is it set?
       jp z,wait           ; no? - keep looping.
       xor a               ; else - reset flag.
       ld (iflag),a
       ret                 ; return.

; --------------------------------------------------------------
; DATA
; --------------------------------------------------------------
; Initial values for the 11 vdp registers.

regdat .db %00000110       ; reg. 0, display and interrupt mode.
                           ; bit 4 = line interrupt (disabled).
                           ; 5 = blank left column (disabled).
                           ; 6 = hori. scroll inhibit (disabled).
                           ; 7 = vert. scroll inhibit (disabled).

       .db %10100001       ; reg. 1, display and interrupt mode.
                           ; bit 0 = zoomed sprites (enabled).
                           ; 1 = 8 x 16 sprites (disabled).
                           ; 5 = frame interrupt (enabled).
                           ; 6 = display (blanked).

       .db $ff             ; reg. 2, name table address.
                           ; $ff = name table at $3800.

       .db $ff             ; reg. 3, n.a.
                           ; always set it to $ff.

       .db $ff             ; reg. 4, n.a.
                           ; always set it to $ff.

       .db $ff             ; reg. 5, sprite attribute table.
                           ; $ff = sprite attrib. table at $3F00.

       .db $ff             ; reg. 6, sprite tile address.
                           ; $ff = sprite tiles in bank 2.

       .db %11110011       ; reg. 7, border color.
                           ; set to color 3 in bank 2.

       .db $00             ; reg. 8, horizontal scroll value = 0.

       .db $00             ; reg. 9, vertical scroll value = 0.

       .db $ff             ; reg. 10, raster line interrupt.
                           ; turn off line int. requests.

cloudmap          .dw $0805 $0805 $0805

initsc .db 30
       .db 36
       .db 200 $33
       .db 200 $b4

level1holemap     .db 6 6 6 6
                  .db 6 6 6 6
                  .db 6 6 6 6

mrultratilemap    .dw $0800
                  .dw $0801
                  .dw $0802
                  .dw $0803

mrultratilemap2   .dw $0800
                  .dw $0801
                  .dw $0802
                  .dw $0803

enemymap          .dw $0804
                  .dw $0810
                  .dw $0811
                  .dw $0812

bgpal  .include "assets\title (palette).inc"
bgtile .include "assets\title (tiles).inc"
bgmap  .include "assets\title (tilemap).inc"

level1pal  .include "assets\level1bg (palette).inc"
level1tile .include "assets\level1bg (tiles).inc"
level1map  .include "assets\level1bg (tilemap).inc"
level1holetile .include "assets\level1hole (tiles).inc"

level2pal  .include "assets\level2bg (palette).inc"
level2tile .include "assets\level2bg (tiles).inc"
level2map  .include "assets\level2bg (tilemap).inc"

level3pal  .include "assets\level3bg (palette).inc"
level3tile .include "assets\level3bg (tiles).inc"
level3map  .include "assets\level3bg (tilemap).inc"

palclouds .include "assets\mrultraanim1a (palette).inc"
palclouds2 .include "assets\mrultralevel2 (palette).inc"

blchar .include "assets\bluechar (tiles).inc"

mrultratile1  .include "assets\mrultraanim1 (tiles).inc"
mrultratile2  .include "assets\mrultraanim2 (tiles).inc"

broccolitile1   .include "assets\broccoli1 (tiles).inc"
broccolitile2   .include "assets\broccoli2 (tiles).inc"

carrottile1   .include "assets\carrot1 (tiles).inc"
carrottile2   .include "assets\carrot2 (tiles).inc"

eggplanttile1   .include "assets\eggplantfinal1 (tiles).inc"
eggplanttile2   .include "assets\eggplantfinal2 (tiles).inc"

timap  .include "assets\title (tilemap).inc"
titile .include "assets\title (tiles).inc"
titune .incbin "assets\titlescreen2.psg"
level1tune .incbin "assets\mrultralevel1.psg"
level2tune .incbin "assets\mrultralevel2.psg"
level3tune .incbin "assets\mrultralevel3.psg"
.include "assets\PSGlib.inc"